@@ -18,6 +18,13 @@
 #define SALTNAME       "[salt]"          // dummy user name of the salt
 #define ADMINNAME      "admin"           // user name of the administrator
 
+typedef struct {                         // type of structure for a record
+  int64_t id;                            // ID of the article
+  int64_t date;                          // date
+  const char *owner;                     // owner
+  const char *text;                      // text
+} COMMENT;
+
 
 /* global variables */
 const char *g_scriptname;                // script name
@@ -34,6 +41,7 @@ int g_searchnum;                         // number of articles in a search page
 int g_listnum;                           // number of articles in a list page
 int g_feedlistnum;                       // number of articles in a RSS feed
 int g_filenum;                           // number of files in a file list page
+int g_sidebarnum;                        // number of items in the side bar
 const char *g_commentmode;               // comment mode
 const char *g_updatecmd;                 // path of the update command
 int g_sessionlife;                       // lifetime of each session
@@ -57,6 +65,7 @@ static bool putfile(TCMPOOL *mpool, const char *path, const char *name,
 static bool outfile(TCMPOOL *mpool, const char *path);
 static TCLIST *searchfiles(TCMPOOL *mpool, const char *expr, const char *order,
                            int max, int skip, bool thum);
+static int comparecomments(const TCLISTDATUM *a, const TCLISTDATUM *b);
 static bool doupdatecmd(TCMPOOL *mpool, const char *mode, const char *baseurl, const char *user,
                         double now, int64_t id, TCMAP *ncols, TCMAP *ocols);
 
@@ -94,6 +103,8 @@ int main(int argc, char **argv){
     g_feedlistnum = tclmax(rp ? tcatoi(rp) : 10, 1);
     rp = tctmplconf(g_tmpl, "filenum");
     g_filenum = tclmax(rp ? tcatoi(rp) : 10, 1);
+    rp = tctmplconf(g_tmpl, "sidebarnum");
+    g_sidebarnum = tclmax(rp ? tcatoi(rp) : 0, 0);
     g_commentmode = tctmplconf(g_tmpl, "commentmode");
     if(!g_commentmode) g_commentmode = "";
     g_updatecmd = tctmplconf(g_tmpl, "updatecmd");
@@ -530,7 +541,7 @@ static void dosession(TCMPOOL *mpool){
       }
       tcmapprintf(vars, "cond", "id:%lld", (long long)p_id);
     } else {
-      TCMAP *cols = tcmapnew2(TINYBNUM);
+      TCMAP *cols = tcmpoolpushmap(mpool, tcmapnew2(TINYBNUM));
       wikiload(cols, p_wiki);
       if(p_mts){
         tcmapprintf(cols, "cdate", "%lld", (long long)now);
@@ -1026,6 +1037,75 @@ static void dosession(TCMPOOL *mpool){
       }
     }
   }
+  if(g_sidebarnum > 0 && strcmp(p_format, "atom")){
+    // side bar
+    TCLIST *res = searcharts(mpool, tdb, NULL, NULL, "cdate", g_sidebarnum, 0, true);
+    int rnum = tclistnum(res);
+    TCLIST *arts = tcmpoollistnew(mpool);
+    for(int i = 0; i < rnum; i++){
+      int64_t id = tcatoi(tclistval2(res, i));
+      TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
+      if(cols){
+        setarthtml(mpool, cols, id, 1, true);
+        tclistpushmap(arts, cols);
+      }
+    }
+    if(tclistnum(arts) > 0) tcmapputlist(vars, "sidearts", arts);
+    res = searcharts(mpool, tdb, NULL, NULL, "xdate", g_sidebarnum, 0, true);
+    rnum = tclistnum(res);
+    TCLIST *coms = tcmpoollistnew(mpool);
+    for(int i = 0; i < rnum; i++){
+      int64_t id = tcatoi(tclistval2(res, i));
+      TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
+      if(cols){
+        rp = tcmapget2(cols, "comments");
+        if(rp && *rp != '\0'){
+          TCLIST *lines = tcmpoolpushlist(mpool, tcstrsplit(rp, "\n"));
+          int cnum = tclistnum(lines);
+          for(int i = 0; i < cnum && i < g_sidebarnum; i++){
+            rp = tclistval2(lines, i);
+            char *co = strchr(rp, '|');
+            if(co){
+              *(co++) = '\0';
+              char *ct = strchr(co, '|');
+              if(ct){
+                *(ct++) = '\0';
+                COMMENT com;
+                memset(&com, 0, sizeof(com));
+                com.id = id;
+                com.date = tcatoi(rp);
+                com.owner = co;
+                com.text = ct;
+                tclistpush(coms, &com, sizeof(com));
+              }
+            }
+          }
+        }
+      }
+    }
+    int cnum = tclistnum(coms);
+    if(cnum > 0){
+      tclistsortex(coms, comparecomments);
+      TCLIST *comments = tcmpoolpushlist(mpool, tclistnew2(cnum));
+      for(int i = 0; i < cnum && i < g_sidebarnum; i++){
+        COMMENT *com = (COMMENT *)tclistval2(coms, i);
+        TCMAP *comment = tcmpoolpushmap(mpool, tcmapnew2(TINYBNUM));
+        tcmapprintf(comment, "id", "%lld", (long long)com->id);
+        char numbuf[NUMBUFSIZ];
+        tcdatestrwww(com->date, INT_MAX, numbuf);
+        tcmapput2(comment, "date", numbuf);
+        tcmapput2(comment, "datesimple", datestrsimple(numbuf));
+        tcmapput2(comment, "owner", com->owner);
+        tcmapput2(comment, "text", com->text);
+        TCXSTR *xstr = tcmpoolxstrnew(mpool);
+        wikitohtmlinline(xstr, com->text, g_scriptname);
+        tcmapput(comment, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
+        tclistpushmap(comments, comment);
+      }
+      tcmapputlist(vars, "sidecoms", comments);
+    }
+    tcmapput2(vars, "sidebar", "true");
+  }
   if(!tctdbclose(tdb)) setdberrmsg(emsgs, tdb, "Closing the database was failed.");
   if(tclistnum(emsgs) > 0) tcmapputlist(vars, "emsgs", emsgs);
   tcmapputmap(vars, "params", params);
@@ -1049,6 +1129,7 @@ static void dosession(TCMPOOL *mpool){
   tcmapput2(vars, "now", numbuf);
   tcdatestrwww(mtime, INT_MAX, numbuf);
   tcmapput2(vars, "mtime", numbuf);
+  tcmapput2(vars, "mtimesimple", datestrsimple(numbuf));
   if(tcmapget2(vars, "prev") || tcmapget2(vars, "next")) tcmapput2(vars, "page", "true");
   tcmapputkeep2(vars, "view", "error");
   tcmapput2(vars, "format", !strcmp(p_format, "atom") ? "atom" : "html");
@@ -1357,6 +1438,18 @@ static TCLIST *searchfiles(TCMPOOL *mpool, const char *expr, const char *order,
 }
 
 
+/* compare two comments by date */
+static int comparecomments(const TCLISTDATUM *a, const TCLISTDATUM *b){
+  COMMENT *coma = (COMMENT *)a->ptr;
+  COMMENT *comb = (COMMENT *)b->ptr;
+  if(coma->date > comb->date) return -1;
+  if(coma->date < comb->date) return 1;
+  if(coma->id > comb->id) return -1;
+  if(coma->id < comb->id) return 1;
+  return strcmp(coma->owner, comb->owner);
+}
+
+
 /* process the update command */
 static bool doupdatecmd(TCMPOOL *mpool, const char *mode, const char *baseurl, const char *user,
                         double now, int64_t id, TCMAP *ncols, TCMAP *ocols){
